-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- Written by Kimberly L. Tripp, SQLskills.com
-- All Rights Reserved.
-- 
-- For more scripts, sample code and additional training resources,
-- go to http://www.SQLskills.com
--	Immerse yourself in SQL Server
--
-- Disclaimer - Thoroughly test this script, execute at your own risk.
-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

USE pubs
go

ALTER TRIGGER employee_insupd
ON Employee
FOR INSERT, UPDATE
AS
DECLARE @RowCount int
SELECT @RowCount = @@RowCount
IF @RowCount > 1
    BEGIN
        RAISERROR('This trigger only handles SINGLE row INSERT and UPDATE statements.', 16, 1)
        ROLLBACK TRANSACTION
        RETURN
    END

--Get the range of level for this job type from the jobs table.
DECLARE @min_lvl 	tinyint,
	   @max_lvl 	tinyint,
	   @emp_lvl 	tinyint,
	   @job_id 	smallint
SELECT @min_lvl = j.min_lvl,
   @max_lvl = j.max_lvl,
   @emp_lvl = i.job_lvl,
   @job_id = i.job_id
FROM dbo.jobs AS j
	JOIN inserted as i ON i.job_id = j.job_id
IF (@job_id = 1) AND (@emp_lvl <> 10)
BEGIN
	RAISERROR ('Job id 1 expects the default level of 10.',16,1)
	ROLLBACK TRANSACTION
	RETURN
END
ELSE
IF (@emp_lvl NOT BETWEEN @min_lvl AND @max_lvl)
BEGIN
	IF (@emp_lvl > @max_lvl)
	BEGIN
	   RAISERROR ('The level supplied: %d is too high. The level for job_id:%d should be between %d and %d.', 16, 1, @emp_lvl, @job_id, @min_lvl, @max_lvl) with log
	   ROLLBACK TRANSACTION
	   RETURN
	END
	ELSE
	BEGIN
	   RAISERROR ('The level for job_id:%d should be between %d and %d.', 16, 1, @job_id, @min_lvl, @max_lvl) 
	   ROLLBACK TRANSACTION
	   RETURN
	END
END
go

select * from jobs
go

insert employee 
	values ('KLT42629F', 'Kimberly', 'L', 'Tripp',
		 13, 27, '0877', '1/1/98')

select * from employee