-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- Written by Kimberly L. Tripp, SQLskills.com
-- All Rights Reserved.
-- 
-- For more scripts, sample code and additional training resources,
-- go to http://www.SQLskills.com
--	Immerse yourself in SQL Server
--
-- Disclaimer - Thoroughly test this script, execute at your own risk.
-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

/* ***************************************************************** **
** This script should be used after the TSQLTutorJoins database has
** been created. The database script was originally used in a series 
** of articles in SQL Server Magazine to describe joins. 		
** ***************************************************************** */
USE TSQLTutorJoins
go

-- General Notes:
-- 1) CustomerIDs 3 & 4 have not made a purchase.
-- 2) No Customer has purchased ProductID 3.
-- 3) 8 total dbo.[Order], 7 to Customer, 1 over-the-counter

-- The following section contains examples of proper
-- join syntax.

-- Inner Joins
-- 6 Customer, 8 dbo.[Order] 
-- INNER JOIN means that we're looking for ONLY the intersection
-- between two tables, maximum number of rows is the larger of the
-- two tables: 8 rows MAX. 

SELECT c.CustomerID 						AS [Customer Number]
		, c.FirstName + ' ' + c.LastName 	AS [Customer Name]
		, o.Quantity
FROM dbo.Customer AS c
	INNER JOIN dbo.[Order] AS o
		ON c.CustomerID = o.CustomerID

-- OLD SQL Syntax
SELECT [Customer Number] = c.CustomerID
			-- using HEADER = expression is still very common
		, c.FirstName + ' ' + c.LastName [Customer Name]
			-- while expression HEADER works - it's often more of a 
			-- mistake (forgetting a comma - watch out)
		, o.Quantity
FROM dbo.Customer c, dbo.[Order] o
WHERE c.CustomerID = o.CustomerID

-- Outer Joins
--6 Customers, 8 Orders
-- OUTER JOIN means that you want to include those rows (from the outer table)
-- which do NOT qualify in the join condition. An outer join is always AFTER the inner
-- join and INCLUDES the row set defined by the inner join.
    	-- If the larger table is the OUTER - then EXACTLY n rows returned 
	-- (where n is the number of rows in the larger of the two tables)

    	-- If the smaller of the two tables is the OUTER - then the maximum can be 	
	-- estimated but the exact number cannot be predicted. 
	-- Imagine ALL rows in the larger table join to ONE row in the smaller (outer)
	-- table... In this case you'd have 1 customer to 8 orders. The inner join would 
	-- return these 8 rows. With the customer table (the smaller table) being the outer
	-- we must return all of the rest of the customers which do not intersect + 5.
	-- In this case the MAXIMUM number of rows is the larger table (if they all join)
	-- plus the smaller table -1 OR 13.

-- Outer table is the SMALLER of two
---------------------------------------------------------------
SELECT c.CustomerID 						AS [Customer Number]
		, c.FirstName + ' ' + c.LastName 	AS [Customer Name]
		, o.Quantity
FROM dbo.[Order] AS o
	RIGHT OUTER JOIN dbo.Customer AS c
		ON c.CustomerID = o.CustomerID

-- LEFT and RIGHT are irrelevant in terms of data when tables are
-- swapped. This query returns the same results as the first; the tables
-- are reversed and the join direction is as well - RIGHT/LEFT.
SELECT c.CustomerID 						AS [Customer Number]
		, c.FirstName + ' ' + c.LastName 	AS [Customer Name]
		, o.Quantity
FROM dbo.Customer AS c
	LEFT OUTER JOIN dbo.[Order] AS o
		ON c.CustomerID = o.CustomerID

-- Outer table is the LARGER of two
---------------------------------------------------------------
SELECT c.CustomerID 						AS [Customer Number]
		, c.FirstName + ' ' + c.LastName 	AS [Customer Name]
		, o.Quantity
FROM dbo.[Order] AS o
	LEFT OUTER JOIN dbo.Customer AS c
		ON c.CustomerID = o.CustomerID

-- OLD Outer Join Syntax 
---------------------------------------------------------------
-- Direction determined by side of * in WHERE clause
-- Do NOT use. Can create "interesting" results... read on.
SELECT c.CustomerID 						AS [Customer Number]
		, c.FirstName + ' ' + c.LastName 	AS [Customer Name]
		, o.Quantity
FROM dbo.[Order] o, dbo.Customer AS c
WHERE o.CustomerID *= c.CustomerID

-- FULL Outer Join
---------------------------------------------------------------
-- ONLY supported with new standard
-- More readable, clear distinction of SARG v JOIN
-- MINIMUM and MAXIMUM are predictable...
-- Minimum is the larger of the two tables IF every row matches a row
-- in BOTH tables. If ALL customers have placed an order then there are 
-- no customers in the outer set. If ALL orders were placed by a valid
-- customer then there are no orders in the outer set. A full outer would
-- be 8 rows. However, if there are no orders placed by valid customers
-- (i.e. all OTC) then the FULL outer would return t1 + t2 rows = 14 MAX.
-- So - for a full outer we have a range of >= smaller table and <= t1 + t2

SELECT c.CustomerID 						AS [Customer Number]
		, c.FirstName + ' ' + c.LastName 	AS [Customer Name]
		, o.Quantity
FROM dbo.[Order] AS o
	FULL OUTER JOIN dbo.Customer AS c
		ON c.CustomerID = o.CustomerID

-- **NO** comparable TSQL Syntax
-- SELECT c.CustomerID 					AS [Customer Number]
-- 		, c.FirstName + ' ' + c.LastName 	AS [Customer Name]
-- 		, o.Quantity
-- FROM dbo.[Order] o, dbo.Customer AS c
-- WHERE c.CustomerID *=* o.CustomerID
-- *=* will NOT work!



-- Adding MORE Tables, MORE Details
---------------------------------------------------------------
SELECT c.CustomerID 						AS [Customer Number]
		, c.FirstName + ' ' + c.LastName 	AS [Customer Name]
		, p.ProductName					AS [Product Name]
		, o.Quantity
FROM dbo.Customer AS c
	INNER JOIN dbo.[Order] AS o ON c.CustomerID = o.CustomerID
	INNER JOIN dbo.Product AS p  ON p.ProductID = o.ProductID

-- Could join these three tables in many ways...

--the same as this...
SELECT c.CustomerID 						AS [Customer Number]
		, c.FirstName + ' ' + c.LastName 	AS [Customer Name]
		, p.ProductName					AS [Product Name]
		, o.Quantity
FROM dbo.[Order] AS o 
	JOIN dbo.Customer AS c ON c.CustomerID = o.CustomerID
	JOIN dbo.Product AS p ON o.ProductID = p.ProductID

--the same as this...
SELECT c.CustomerID 						AS [Customer Number]
		, c.FirstName + ' ' + c.LastName 	AS [Customer Name]
		, p.ProductName					AS [Product Name]
		, o.Quantity
FROM dbo.Product AS p 
	JOIN dbo.[Order] AS o ON o.ProductID = p.ProductID
	JOIN dbo.Customer AS c ON c.CustomerID = o.CustomerID

--the same as this...
SELECT c.CustomerID 						AS [Customer Number]
		, c.FirstName + ' ' + c.LastName 	AS [Customer Name]
		, p.ProductName					AS [Product Name]
		, o.Quantity
FROM dbo.[Order] AS o 
	JOIN dbo.product AS p ON o.ProductID = p.ProductID
	JOIN dbo.Customer AS c ON c.CustomerID = o.CustomerID

-- What about outer join?
SELECT c.CustomerID AS [Customer Number]
	, c.FirstName + ' ' + c.LastName AS [Customer Name]
	, p.ProductName AS [Product Name]
	, o.Quantity
FROM dbo.Customer AS c
	RIGHT OUTER JOIN dbo.[Order] AS o 
		ON c.CustomerID = o.CustomerID
	INNER JOIN dbo.Product AS p
		ON p.ProductID = o.ProductID

-- What happens here?
SELECT c.CustomerID 						AS [Customer Number]
		, c.FirstName + ' ' + c.LastName 	AS [Customer Name]
		, p.ProductName					AS [Product Name]
		, o.Quantity
FROM dbo.Customer AS c
	LEFT OUTER JOIN dbo.[Order] AS o 
		ON c.CustomerID = o.CustomerID
	 JOIN dbo.Product AS p 
		ON o.ProductID = p.ProductID

-- To get Customers 3 and 4 you need to continue keeping the "LEFT" set 
-- through to the next join. The INNERsection loses those rows as they
-- don't have a matching row in the product table. 
SELECT c.CustomerID 						AS [Customer Number]
		, c.FirstName + ' ' + c.LastName 	AS [Customer Name]
		, p.ProductName					AS [Product Name]
		, o.Quantity
FROM dbo.Customer AS c
	 LEFT OUTER JOIN dbo.[Order] AS o 
		ON c.CustomerID = o.CustomerID
	 LEFT JOIN dbo.Product AS p 
		ON o.ProductID = p.ProductID

-- Add a fourth table...
SELECT c.CustomerID AS [Customer Number]
	, c.FirstName + ' ' + c.LastName AS [Customer Name]
	, p.ProductName AS [Product Name]
	, o.Quantity
FROM dbo.Customer AS c
	INNER JOIN dbo.[Order] AS o 
		ON c.CustomerID = o.CustomerID
	INNER JOIN dbo.Product AS p
		ON p.ProductID = o.ProductID
	INNER JOIN dbo.Category AS Cat
		ON Cat.CategoryID = p.CategoryID
WHERE cat.CategoryName = 'Water-related'

-- New Outer join syntax vs OLD outer join syntax
-- NEW Outer Join Syntax is:
-- 	1) Easier to read
-- 	2) Less error prone 
--	3) More flexible with regard to WHEN 
--		your search arguments are applied.

-- Search Arguments can be applied in one of two ways:
-- Option1
-- INNER JOIN
-- SEARCH CONDITION (in the JOIN CLAUSE)
-- OUTER JOIN

-- Option2
-- INNER JOIN
-- OUTER JOIN
-- SEARCH CONDITION (in the WHERE clause)

-- With the Old Outer Join Syntax you ALWAYS get Option1
SELECT c.CustomerID 						AS [Customer Number]
		, c.FirstName + ' ' + c.LastName 	AS [Customer Name]
		, o.Quantity
FROM dbo.[Order] o, dbo.Customer AS c
WHERE c.CustomerID *= o.CustomerID AND o.Quantity IS NULL

-- With the NEW Outer Join Syntax you GET A CHOICE!!! 
-- To get Option1 supply the search argument as part of the join
-- in the ON Clause
SELECT c.CustomerID 						AS [Customer Number]
		, c.FirstName + ' ' + c.LastName 	AS [Customer Name]
		, o.Quantity
FROM dbo.Customer AS c
	LEFT OUTER JOIN dbo.[Order] AS o
		ON c.CustomerID = o.CustomerID AND o.Quantity IS NULL

-- To get Option2 supply the search argument as a
-- regular WHERE Clause
SELECT c.CustomerID 						AS [Customer Number]
		, c.FirstName + ' ' + c.LastName 	AS [Customer Name]
		, o.Quantity
FROM dbo.Customer AS c
	LEFT OUTER JOIN dbo.[Order] AS o
		ON c.CustomerID = o.CustomerID 
WHERE o.Quantity IS NULL

-- Therefore - the New Standard Syntax 
-- is more flexible!

-- Old SQL Syntax Cross Join (tableA * tableB)
SELECT c.CustomerID 						AS [Customer Number]
		, c.FirstName + ' ' + c.LastName 	AS [Customer Name]
		, o.Quantity
FROM dbo.Customer c, dbo.[Order] o

-- New SQL Syntax Cross Join
SELECT c.CustomerID 						AS [Customer Number]
		, c.FirstName + ' ' + c.LastName 	AS [Customer Name]
		, o.Quantity
FROM dbo.Customer AS c
	CROSS JOIN dbo.[Order] AS o

-- Useful Cross Join
-- authors firstnames * authors lastnames * middleinitials
-- SELECT 23*23*23
-- INSERT tname
SELECT DISTINCT a1.au_fname AS FirstName
	, a2.au_lname AS LastName
	, LEFT(a3.city, 1) AS MiddleInitial
FROM pubs.dbo.authors AS a1
	CROSS JOIN pubs.dbo.authors AS a2
	CROSS JOIN pubs.dbo.authors AS a3

